@@ -8,16 +8,16 @@
 #include "mimetypedata.h"
 #include "mimetypewriter.h"
 #include <KApplicationTrader>
+#include <KConfigGroup>
 #include <KParts/PartLoader>
+#include <KProtocolManager>
+#include <KService>
+#include <KSharedConfig>
+#include <QDebug>
 #include <QFileInfo>
 #include <QMimeDatabase>
 #include <QStandardPaths>
 #include <QXmlStreamReader>
-#include <kconfiggroup.h>
-#include <kprotocolmanager.h>
-#include <kservice.h>
-#include <ksharedconfig.h>
-#include <qdebug.h>
 
 MimeTypeData::MimeTypeData(const QString &major)
     : m_askSave(AskSaveDefault)
@@ -223,6 +223,7 @@ QStringList MimeTypeData::getAppOffers() const
 QStringList MimeTypeData::getPartOffers() const
 {
     QStringList servicesIds;
+    // We do not do any sorting, because KParts uses the order in which the entries are saved
     const auto partOfferList = KParts::PartLoader::partsForMimeType(name());
     for (const auto &metaData : partOfferList) {
         servicesIds.append(metaData.pluginId());
@@ -233,7 +234,7 @@ QStringList MimeTypeData::getPartOffers() const
 void MimeTypeData::getMyServiceOffers() const
 {
     m_appServices = getAppOffers();
-    m_embedServices = getPartOffers();
+    m_embedParts = getPartOffers();
     m_bFullInit = true;
 }
 
@@ -245,12 +246,12 @@ QStringList MimeTypeData::appServices() const
     return m_appServices;
 }
 
-QStringList MimeTypeData::embedServices() const
+QStringList MimeTypeData::embedParts() const
 {
     if (!m_bFullInit) {
         getMyServiceOffers();
     }
-    return m_embedServices;
+    return m_embedParts;
 }
 
 bool MimeTypeData::isMimeTypeDirty() const
@@ -394,13 +395,17 @@ void MimeTypeData::syncServices()
         saveRemovedServices(removedApps, m_appServices, oldAppServices);
     }
 
+    KSharedConfig::Ptr kpartsProfile = KSharedConfig::openConfig(QStringLiteral("kpartsrc"), KConfig::NoGlobals);
     const QStringList oldPartServices = getPartOffers();
-    if (oldPartServices != m_embedServices) {
-        // Handle removed services
-        KConfigGroup addedParts(profile, "Added KDE Service Associations");
-        saveServices(addedParts, m_embedServices);
-        KConfigGroup removedParts(profile, "Removed KDE Service Associations");
-        saveRemovedServices(removedParts, m_embedServices, oldPartServices);
+    if (oldPartServices != m_embedParts) {
+        KConfigGroup addedParts(kpartsProfile, "Added KDE Part Associations");
+        if (m_embedParts.isEmpty()) {
+            addedParts.deleteEntry(name());
+        } else {
+            addedParts.writeXdgListEntry(name(), m_embedParts);
+        }
+        KConfigGroup removedParts(kpartsProfile, "Removed KDE Part Associations");
+        saveRemovedServices(removedParts, m_embedParts, oldPartServices);
     }
 
     // Clean out any kde-mimeapps.list which would take precedence any cancel our changes.
@@ -567,12 +572,12 @@ void MimeTypeData::setAppServices(const QStringList &dsl)
     m_appServicesModified = true;
 }
 
-void MimeTypeData::setEmbedServices(const QStringList &dsl)
+void MimeTypeData::setEmbedParts(const QStringList &dsl)
 {
     if (!m_bFullInit) {
         getMyServiceOffers(); // so that m_bFullInit is true
     }
-    m_embedServices = dsl;
+    m_embedParts = dsl;
     m_embedServicesModified = true;
 }
 
